#ifndef _MAIN_H_
#define _MAIN_H_
/* === COPYRIGHT 2000 (C) ========================= TOM HENDRICK ===

   FILE:                                                      main.h

   DESCRIPTION:
      Outlines the main data structures for the main module Tom's IPX
      tunneling daemon.  Also outlines the function prototyeps for
      utility functions within the main module, such as usage and help
      information as well as the main control structure.

   $REVISION:$
   
   LICENSE:
      This file, and the files included within the tipxd project are
      distributed under the GNU Public License(GPL).  Please read the
      LICENSE file included with this distribution or consult the license
      on the web at http://www.opensource.org/licenses/gpl-license.html

   =========== T I P X D . S O U R C E F O R G E . N E T =========== */


/* -----------------------------------------------------------------
   PREPROCESSOR INCLUDES
   ----------------------------------------------------------------- */
#include <stdio.h>

#include "ether.h"
#include "router.h"
#include "peer.h"

/* -----------------------------------------------------------------
   STATIC DATA AND GLOBAL CONSTANTS
   ----------------------------------------------------------------- */
#define LOG_LOCAL_PACKET   0x01
#define LOG_LOCAL_CONTROL  0x02
#define LOG_REMOTE_PACKET  0x10
#define LOG_REMOTE_CONTROL 0x20
#define LOG_REMOTE_ADMIN   0x30

#define OPT_STDERR         0x01
#define OPT_VERBOSE        0x02

/* -----------------------------------------------------------------
   PUBLIC DATA TYPES
   ----------------------------------------------------------------- */

/*
 * This is the main structure for the system information that is used by
 * the entire program.  There should only be one of these at any given
 * time, allocationed by main() when the program initializes.
 */
typedef struct _sysinfo_t {
   routerinfo_t *routerinfo;
   etherinfo_t  *etherinfo;
   peerinfo_t   *peerinfo;
   selectinfo_t *udp_data_selectinfo;
   selectinfo_t *udp_ctrl_selectinfo;
   unsigned short data_listen_port;
   unsigned short control_listen_port;
   unsigned short admin_listen_port;
   unsigned int data_socket_type;
   unsigned int control_socket_type;
   unsigned int admin_socket_type;
   unsigned short log_flags;
   unsigned short opt_flags;
   const char *config_filename;
   char *log_filename;
   char *site_string;
   FILE       *log_fp;
   int         argc;
   char      **argv;
} sysinfo_t;

/* -----------------------------------------------------------------
   GLOBAL AND OTHERWISE EXPORTED DATA
   ----------------------------------------------------------------- */

/*
 * This is the one true sysinfo, and the only one that should exist
 * throughout the life of the program's execution.
 */
extern sysinfo_t sysinfo;

/* -----------------------------------------------------------------
   PUBLIC FUNCTION PROTOTYPES
   ----------------------------------------------------------------- */

/* Verify the components of the sysinfo structure */
extern int verify_sysinfo( void );

extern int  main    ( int argc, char *argv[] );
extern void sig_int ( int signo );
extern void sig_hup ( int signo );
extern void sig_usr1( int signo );

/* from the lex tokenizer */
extern FILE *yyin;

#endif /* _MAIN_H_ */
