#ifndef _PEER_H_
#define _PEER_H_
/* === COPYRIGHT 2000 (C) ========================= TOM HENDRICK ===

   FILE:                                                      peer.h

   DESCRIPTION:
      Defines data structures and operation prototypes necessary to manage
      the registration and filtering of IPX peers.

   $REVISION:$
   
   LICENSE:
      This file, and the files included within the tipxd project are
      distributed under the GNU Public License(GPL).  Please read the
      LICENSE file included with this distribution or consult the license
      on the web at http://www.opensource.org/licenses/gpl-license.html

   =========== T I P X D . S O U R C E F O R G E . N E T =========== */


/* -----------------------------------------------------------------
   PREPROCESSOR INCLUDES
   ----------------------------------------------------------------- */
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#include "router.h"
#include "ether.h"

/* -----------------------------------------------------------------
   STATIC DATA AND GLOBAL CONSTANTS
   ----------------------------------------------------------------- */

/* Frametype filter constants */
#define FILTER_802_3       0x01
#define FILTER_802_2       0x02
#define FILTER_SNAP        0x04
#define FILTER_II          0x08
/* Frametype broadcast filter constants */
#define FILTER_802_3_BCAST 0x10
#define FILTER_802_2_BCAST 0x20
#define FILTER_SNAP_BCAST  0x40
#define FILTER_II_BCAST    0x80
/* Constants for peer location */
#define PEER_LOC_UNREGISTERED 0
#define PEER_LOC_LOCAL        1
#define PEER_LOC_REMOTE       2

/* -----------------------------------------------------------------
   PUBLIC DATA TYPES
   ----------------------------------------------------------------- */

/*
 * This structure defines the data necessary to keep track of an host on
 * the IPX network.  If the fitlering flags are set, packets from the
 * appropriate frametype will be dropped.
 */
typedef struct _peerinfo_t {
   unsigned char address[ETHER_ADDR_LEN];
   unsigned char location;
   unsigned int  filters;
   union {
      etherinfo_t *ether;
      routerinfo_t *router;
   } destination;

   // Linked List
   struct _peerinfo_t *next;
} peerinfo_t;

/* -----------------------------------------------------------------
   GLOBAL AND OTHERWISE EXPORTED DATA
   ----------------------------------------------------------------- */


/* -----------------------------------------------------------------
   PUBLIC FUNCTION PROTOTYPES
   ----------------------------------------------------------------- */

/* Dump these to a logfile in human-readable format */
extern void dump_peer( FILE *fp, peerinfo_t *info );

/* Send to a file descriptor in machine-readable format */
extern void send_peer( int fd, peerinfo_t *info );

/* List functions */
extern peerinfo_t *find_peer( char * );
extern void add_peer( peerinfo_t *peer );
extern void route( peerinfo_t *, peerinfo_t *, void *, char *, size_t );
extern void send_rmt_all( routerinfo_t *, char *, size_t );
extern void send_ipx_all( etherinfo_t *, char *, size_t );

#endif /* _PEER_H_ */
