#ifndef _ROUTER_H_
#define _ROUTER_H_
/* === COPYRIGHT 2000 (C) ========================= TOM HENDRICK ===

   FILE:                                                    router.h

   DESCRIPTION:
      Defines the function prototypes and data structures for dealing
      with connections to remote routers.

   $REVISION:$
   
   LICENSE:
      This file, and the files included within the tipxd project are
      distributed under the GNU Public License(GPL).  Please read the
      LICENSE file included with this distribution or consult the license
      on the web at http://www.opensource.org/licenses/gpl-license.html

   =========== T I P X D . S O U R C E F O R G E . N E T =========== */


/* -----------------------------------------------------------------
   PREPROCESSOR INCLUDES
   ----------------------------------------------------------------- */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "select.h"

/* -----------------------------------------------------------------
   STATIC DATA AND GLOBAL CONSTANTS
   ----------------------------------------------------------------- */
#define MAX_HOSTNAME 32

#define ROUTER_CONNECT   0x1
#define ROUTER_ACCEPT    0x2
#define ROUTER_CONNECTED 0x4

/* -----------------------------------------------------------------
   PUBLIC DATA TYPES
   ----------------------------------------------------------------- */

/*
 * This structure defines the information for connections to remote
 * instances of tipxd.
 */
typedef struct _routerinfo_t {
   unsigned long       id;
   unsigned char       hostname[MAX_HOSTNAME];
   unsigned int        flags;
   struct sockaddr_in  hostaddr;
   struct hostent     *host_entry;
   unsigned long       bytes_rcvd;
   unsigned long       bytes_sent;
   selectinfo_t       *data_selectinfo;
   selectinfo_t       *ctrl_selectinfo;
   unsigned short      data_port;
   unsigned short      ctrl_port;
   unsigned int        data_socket_type;
   unsigned int        ctrl_socket_type;
   unsigned long       sequence;
   struct _routerinfo_t *next;
} routerinfo_t;

/*
 * This is a header sent before every packet data.
 */
typedef struct _datapkt_t {
   unsigned long sequence;
   unsigned long payload_size;
} datapkt_t;

/* -----------------------------------------------------------------
   GLOBAL AND OTHERWISE EXPORTED DATA
   ----------------------------------------------------------------- */


/* -----------------------------------------------------------------
   PUBLIC FUNCTION PROTOTYPES
   ----------------------------------------------------------------- */

/* Verify the contents of the routerinfo structure */
extern int verify_router( routerinfo_t *info );

/* Dump these to a logfile in human-readable format */
extern void dump_router( FILE *fp, routerinfo_t *info );

/* Send to a file descriptor in machine-readable format */
extern void send_router( int fd, routerinfo_t *info );

/* Add a router to the router list */
extern void add_router( routerinfo_t *info );

/* These are socket operations for DATA sockets */
extern void open_data_udp ( struct _selectinfo_t *, void * );
extern void write_data_udp( struct _selectinfo_t *, void *, char *, size_t );
extern void read_data_udp ( struct _selectinfo_t *, void *);

extern void open_ctrl_udp ( struct _selectinfo_t *, void * );
extern void write_ctrl_udp( struct _selectinfo_t *, void *, char *, size_t );

/* We have no special requirements for closing or errors, so we can use the
 * generalized functions from select.c
 */
#define error_data_udp error_general
#define close_data_udp close_general

#define error_ctrl_udp error_general
#define close_ctrl_udp close_general

#endif /* _ROUTER_H_ */
