#ifndef _SELECT_H_
#define _SELECT_H_
/* === COPYRIGHT 2000 (C) ========================= TOM HENDRICK ===

   FILE:                                                    select.h

   DESCRIPTION:
      Contains the data structure types that are going to be used for
      select() syscall.  Each of these handles a particular file descriptor
      and contains all the operations and information necessary to take
      care of individual descriptors.

   $REVISION$
   
   LICENSE:
      This file, and the files included within the tipxd project are
      distributed under the GNU Public License(GPL).  Please read the
      LICENSE file included with this distribution or consult the license
      on the web at http://www.opensource.org/licenses/gpl-license.html

   =========== T I P X D . S O U R C E F O R G E . N E T =========== */


/* -----------------------------------------------------------------
   PREPROCESSOR INCLUDES
   ----------------------------------------------------------------- */
#include <stdio.h>

/* -----------------------------------------------------------------
   STATIC DATA AND GLOBAL CONSTANTS
   ----------------------------------------------------------------- */
#define SELECT_TYPE_CTRL_UDP 0x01
#define SELECT_TYPE_CTRL_TCP 0x02
#define SELECT_TYPE_ADMN_UDP 0x03
#define SELECT_TYPE_ADMN_TCP 0x04
#define SELECT_TYPE_DATA_UDP 0x05
#define SELECT_TYPE_DATA_TCP 0x06
#define SELECT_TYPE_IPX      0x07

/* -----------------------------------------------------------------
   PUBLIC DATA TYPES
   ----------------------------------------------------------------- */

struct _selectinfo_t;
/* Typedefs for function pointers */
typedef void (*read_operation) (struct _selectinfo_t *, void *);
typedef void (*write_operation)(struct _selectinfo_t *, void *, char *, size_t);
typedef void (*error_operation)(struct _selectinfo_t *, void *, int);
typedef void (*open_operation) (struct _selectinfo_t *, void *);
typedef void (*close_operation)(struct _selectinfo_t *, void *);

/*
 * This is the structure used for handling the list of file descriptors and
 * forming them into an FD_SET for the select call.  Each record represents
 * a single file descriptor, and the functions point to functions which
 * perform operations on file descriptors of the appropriate type.
 */
typedef struct _selectinfo_t {
   int fd;
   int type;
   read_operation  read_op;
   write_operation write_op;
   error_operation error_op;
   open_operation  open_op;
   close_operation close_op;
   void *data;  /* This is a pointer to the ether/routerinfo */
   void *arg;   /* Specific data for the function call */
   struct _selectinfo_t *next;
} selectinfo_t;

/* -----------------------------------------------------------------
   GLOBAL AND OTHERWISE EXPORTED DATA
   ----------------------------------------------------------------- */

/*
 * This is the one true sysinfo, and the only one that should exist
 * throughout the life of the program's execution.
 */
extern selectinfo_t *selectinfo;

/* -----------------------------------------------------------------
   PUBLIC FUNCTION PROTOTYPES
   ----------------------------------------------------------------- */

/* Dump these to a FILE * in human-readable format */
extern void dump_select( FILE *fp, selectinfo_t *info );

/* Add a selectinfo to the system list */
extern void add_select ( selectinfo_t * );

/* These are generalize operations that alot of the socket types have in
 * common
 */
extern void error_general( struct _selectinfo_t *, void *, int );
extern void close_general( struct _selectinfo_t *, void * );
extern void form_select_list( void );

#endif /* _SELECT_H_ */

