%{
/* === COPYRIGHT 2000 (C) ========================= TOM HENDRICK ===

   FILE:                                                      conf.l

   DESCRIPTION:
      Defines the tokenizer for reading the configuration file.

   $REVISION:$
   
   LICENSE:
      This file, and the files included within the tipxd project are
      distributed under the GNU Public License(GPL).  Please read the
      LICENSE file included with this distribution or consult the license
      on the web at http://www.opensource.org/licenses/gpl-license.html

   =========== T I P X D . S O U R C E F O R G E . N E T =========== */

/* -----------------------------------------------------------------
   PREPROCESSOR INCLUDES
   ----------------------------------------------------------------- */
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <syslog.h>
#include <sys/socket.h>

#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>

#include <signal.h>

#include "main.h"

peerinfo_t peer;
etherinfo_t ether;
routerinfo_t router;
int len = 0;
int i = 0;
char *p = NULL;
int retval;

int extract_decimal( char *, unsigned short* );
int extract_quoted_string( char *, char ** );
int extract_ether_address( char *, char *);
%}

%s PEER DEVICE ROUTER GENERAL
%%
[ \t]+ { }
"#" { /* Munch comment characters */ }

"BEGIN GENERAL" { BEGIN GENERAL; }

<GENERAL>"DATA"[ \t]+"udp/"[0-9]+ {
   retval = extract_decimal( yytext, &(sysinfo.data_listen_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      sysinfo.data_socket_type = SOCK_DGRAM;
   }
}

<GENERAL>"DATA"[ \t]+"tcp/"[0-9]+ {
   retval = extract_decimal( yytext, &(sysinfo.data_listen_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      sysinfo.data_socket_type = SOCK_STREAM;
   }
}

<GENERAL>"CTRL"[ \t]+"udp/"[0-9]+ {
   retval = extract_decimal( yytext, &(sysinfo.control_listen_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      sysinfo.control_socket_type = SOCK_DGRAM;
   }
}

<GENERAL>"CTRL"[ \t]+"tcp/"[0-9]+ {
   retval = extract_decimal( yytext, &(sysinfo.control_listen_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal" );
   }
   else
   {
      sysinfo.control_socket_type = SOCK_STREAM;
   }
}

<GENERAL>"ADMN"[ \t]+"udp/"[0-9]+ {
   retval = extract_decimal( yytext, &(sysinfo.admin_listen_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal" );
   }
   else
   {
      sysinfo.admin_socket_type = SOCK_DGRAM;
   }
}

<GENERAL>"ADMN"[ \t]+"tcp/"[0-9]+ {
   retval = extract_decimal( yytext, &(sysinfo.admin_listen_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal" );
   }
   else
   {
      sysinfo.admin_socket_type = SOCK_STREAM;
   }
}

<GENERAL>"LOGF"[ \t]+\".+\" {
   retval = extract_quoted_string( yytext, &(sysinfo.log_filename) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract quoted string\n" );
   }
}

<GENERAL>"SITE"[ \t]+\".+\" {
   retval = extract_quoted_string( yytext, &(sysinfo.site_string) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract quoted string\n" );
   }
}

<GENERAL>"LOGV"[ \t]+\".+\" { retval = extract_quoted_string( yytext, &p );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract quoted string\n" );
   }
   else
   {
      len = strlen( p );
      sysinfo.log_flags = 0;
      for( i = 0; i < len; i++ )
      {
       switch(p[i])
       {
        case '\"': break;
        case 'c':
          sysinfo.log_flags |= LOG_LOCAL_CONTROL;
          break;
        case 'C':
          sysinfo.log_flags |= LOG_REMOTE_CONTROL;
          break;
        case 'p':
          sysinfo.log_flags |= LOG_LOCAL_PACKET;
          break;
        case 'P':
          sysinfo.log_flags |= LOG_REMOTE_PACKET;
          break;
        case 'a':
          sysinfo.log_flags |= LOG_REMOTE_ADMIN;
          break;
        default:
          fprintf( stderr, "Unrecognized directive %c"
                   " skipping\n", p[i] );
         }
      }
   }
}

<GENERAL>"END" {
   BEGIN INITIAL;
   retval = verify_sysinfo( );
   if( retval < 0 )
   {
      printf( "GENERAL section incomplete!\n" );
      exit( -1 );
   }
}

"BEGIN PEER" {
   BEGIN PEER;
   memset( &peer, '\0', sizeof(peerinfo_t) );
}

<PEER>"ADDR"[ \t]+[0-9A-Fa-f:]+ {
   retval = extract_ether_address( &yytext[4], peer.address );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract field ADDR\n" );
   }
}

<PEER>"FTYP"[ \t]+"802.3" {
   peer.filters |= FILTER_802_3;
}
<PEER>"BTYP"[ \t]+"802.3" {
   peer.filters |= FILTER_802_3_BCAST;
}
<PEER>"FTYP"[ \t]+"802.2" {
   peer.filters |= FILTER_802_2;
}
<PEER>"BTYP"[ \t]+"802.2" {
   peer.filters |= FILTER_802_2_BCAST;
}
<PEER>"FTYP"[ \t]+"SNAP" {
   peer.filters |= FILTER_SNAP;
}
<PEER>"BTYP"[ \t]+"SNAP" {
   peer.filters |= FILTER_SNAP_BCAST;
}
<PEER>"FTYP"[ \t]+"II" {
   peer.filters |= FILTER_II;
}
<PEER>"BTYP"[ \t]+"II" {
   peer.filters |= FILTER_II_BCAST;
}

<PEER>"END" {
   BEGIN INITIAL;
   {
      peerinfo_t *peerinfo = (peerinfo_t *)malloc( sizeof(peerinfo_t) );
      memset( peerinfo, '\0', sizeof(peerinfo_t) );
      memcpy( peerinfo, &peer, sizeof( peerinfo_t ) );
      
      add_peer( peerinfo );
   }
}

"BEGIN ROUTER" {
   BEGIN ROUTER;
   memset( &router, '\0', sizeof( routerinfo_t ) );
}
<ROUTER>"ADDR"[ \t]+\".+\" {
   retval = extract_quoted_string( yytext, &p );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract quoted string\n" );
   }
   else
   {
      memset( router.hostname, '\0', MAX_HOSTNAME );
      strncpy( router.hostname, p, MAX_HOSTNAME - 1 );
   }
}

<ROUTER>"CTCT"[ \t]+"connect" {
   router.flags = ROUTER_CONNECT;
}
<ROUTER>"CTCT"[ \t]+"accept" {
   router.flags = ROUTER_ACCEPT;
}

<ROUTER>"DATA"[ \t]+"udp/"[0-9]+ {
   retval = extract_decimal( yytext, &(router.data_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      router.data_socket_type = SOCK_DGRAM;
   }
}

<ROUTER>"DATA"[ \t]+"tcp/"[0-9]+ {
   retval = extract_decimal( yytext, &(router.data_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      router.data_socket_type = SOCK_STREAM;
   }
}

<ROUTER>"CTRL"[ \t]+"udp/"[0-9]+ {
   retval = extract_decimal( yytext, &(router.ctrl_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      router.ctrl_socket_type = SOCK_DGRAM;
   }
}

<ROUTER>"CTRL"[ \t]+"tcp/"[0-9]+ {
   retval = extract_decimal( yytext, &(router.data_port) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
   else
   {
      router.ctrl_socket_type = SOCK_STREAM;
   }
}

<ROUTER>"END" {
   BEGIN INITIAL;
   retval = verify_router( &router );
   if( retval > -1 )
   {
      
      routerinfo_t *routerinfo = (routerinfo_t *)malloc( sizeof(routerinfo_t) );
      memset( routerinfo, '\0', sizeof(routerinfo_t) );
      memcpy( routerinfo, &router, sizeof( routerinfo_t ) );
      routerinfo->data_selectinfo = NULL;
      routerinfo->ctrl_selectinfo = NULL;
      
      add_router( routerinfo );
   }
}

"BEGIN DEVICE" {
   BEGIN DEVICE;
   memset( &ether, '\0', sizeof( etherinfo_t ) );
}

<DEVICE>"NAME"[ \t]+\".+\" {
   retval = extract_quoted_string( yytext, &p );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract quoted string\n");
   }
   else
   {
      strncpy( ether.name, p, 15 );
      free( p );
   }
}

<DEVICE>"INDX"[ \t]+[0-9]+ {
   retval = extract_decimal( yytext, &(ether.ifindex) );
   if( retval < 0 )
   {
      fprintf( stderr, "Unable to extract decimal\n" );
   }
}

<DEVICE>"FTYP"[ \t]+"802.3" {
   ether.protomask |= PROTO_802_3_MASK;
}

<DEVICE>"FTYP"[ \t]+"802.2" {
   ether.protomask |= PROTO_802_2_MASK;
}

<DEVICE>"FTYP"[ \t]+"snap" {
   ether.protomask |= PROTO_SNAP_MASK;
}

<DEVICE>"FTYP"[ \t]+"ii" {
   ether.protomask |= PROTO_II_MASK;
}

<DEVICE>"END" {
      {
      int mask = 0;
      etherinfo_t *etherinfo = (etherinfo_t*)malloc(sizeof(etherinfo_t));
      memset( etherinfo, '\0', sizeof(etherinfo_t));
      memcpy( etherinfo, &ether, sizeof( etherinfo_t ) );

      etherinfo->next = NULL;

      /* We need to see how many protos to open */
      i = 0;
      mask = etherinfo->protomask;
      while( mask != 0 )
      {
         i += (mask & 0x01);
         mask = mask >> 1;
      }

      if( i == 0 )
      {
         /* No protocols, ignore this device entry */
         fprintf( stderr, "No protocols found for device!  Ignoring...\n");
         free( etherinfo );
      }
      else
      {
         int j = 0;
         /* Allocate an array of the correct size */
         etherinfo->protocols =
               (protoinfo_t*)malloc(sizeof(protoinfo_t) * i);

         /* Okay, now look for the specific protocols and create them */
         if( etherinfo->protomask & PROTO_802_3_MASK )
         {
            etherinfo->protocols[j].protocol = ETH_P_802_3;
            etherinfo->protocols[j].flags = PROTO_802_3_MASK;
            strncpy( etherinfo->protocols[j].name, "802.3", 15 );
            etherinfo->protocols[j].bytes_rcvd = 0;
            etherinfo->protocols[j].selectinfo = NULL;

            j++;
         }
         if( etherinfo->protomask & PROTO_802_2_MASK )
         {
            etherinfo->protocols[j].protocol = ETH_P_802_2;
            etherinfo->protocols[j].flags = PROTO_802_2_MASK;
            strncpy( etherinfo->protocols[j].name, "802.2", 15 );
            etherinfo->protocols[j].bytes_rcvd = 0;
            etherinfo->protocols[j].selectinfo = NULL;

            j++;
         }
         if( etherinfo->protomask & PROTO_SNAP_MASK )
         {
            etherinfo->protocols[j].protocol = ETH_P_SNAP;
            etherinfo->protocols[j].flags = PROTO_SNAP_MASK;
            strncpy( etherinfo->protocols[j].name, "SNAP", 15 );
            etherinfo->protocols[j].bytes_rcvd = 0;
            etherinfo->protocols[j].selectinfo = NULL;

            j++;
         }
         if( etherinfo->protomask & PROTO_II_MASK )
         {
            etherinfo->protocols[j].protocol = ETH_P_IPX;
            etherinfo->protocols[j].flags = PROTO_II_MASK;
            strncpy( etherinfo->protocols[j].name, "II", 15 );
            etherinfo->protocols[j].bytes_rcvd = 0;
            etherinfo->protocols[j].selectinfo = NULL;

            j++;
         }

         /* Add the etherinfo */
         etherinfo->protocount = i;
         if( verify_ether( etherinfo ) > -1 )
            add_ether( etherinfo );
      }
   }
}
%%
int yywrap( )
{
   return 1;
}

/* -----------------------------------------------------------------

   FUNCTION:                                      extract_decimal( )

   DESCRIPTION:
      Takes the given string, searches it for the first digit character,
      and extracts a number from it.  It treats the digits as decimal, and
      places the result into the int *val argument.  If no decimal number
      can be extracted, then this function returns -1, and the value of val
      is unspecified.

   ARGUMENTS:
      char *s - the string to extract from.

   RETURN VALUE:
      0 on success, -1 on error.

   ----------------------------------------------------------------- */
int extract_decimal( char *s, unsigned short *val )
{
   size_t len;
   char *p;
   int i;

   /* Sanity check these pointers first */
   if( ( val == NULL ) || ( s == NULL ) )
   {
      return -1;
   }

   /* Get the string length */
   len = strlen( s );

   /* Now, find the first digit in s */
   for( i = 0; i < len; i++ )
   {
      if( isdigit( s[i] ) )
      {
         /* Extract the number starting from here */
         *val = strtoul( &s[i], &p, 0 );
         /* Check for errors */
         if( p != &s[i] )
         {
            /* Some of the string was valid, so return */
            return 0;
         }
      }
   }

   /* If we got here, there was nothing in the string worth messing with
    * so there is an error */
   return -1;
}

/* -----------------------------------------------------------------

   FUNCTION:                                extract_quoted_string( )

   DESCRIPTION:
      Takes the given string, and extracts the first quoted string from it,
      and returns a pointer to a newly allocated buffer containing the
      string enclosed within quotes, without the quote characters.

   ARGUMENTS:
      char *s - the string to extract from.
      char **val - the quoted string

   RETURN VALUE:
      0 on success, -1 on error.

   ----------------------------------------------------------------- */
int extract_quoted_string( char *s, char **val )
{
   int len, l;
   char *p = NULL;
   int i;

   /* Sanity check these pointers first */
   if( ( val == NULL ) || ( s == NULL ) )
   {
      return -1;
   }

   /* Get the string length */
   len = strlen( s );
   l = -1;

   /* Now, find the first digit in s */
   for( i = 0; i < len; i++ )
   {
      if( s[i] == '\"' )
      {
         /* Are we starting the quoted string? */
         if( l == -1 )
         {
            l++;
         }
         /* Are we ending the quoted string? */
         else
         {
            *val = (char *)malloc( l + 1 );
            memset( *val, '\0', l+1);
            strncpy( *val, p, l );

            return 0;
         }
      }
      else
      {
         /* If we are in the quoted string, this is one more char */
         if( l == 0 )
         {
            p = &s[i];
         }
         if( l > -1 )
         {
            l++;
         }
      }
   }

   /* If we got here, there was nothing in the string worth messing with
    * so there is an error */
   return -1;
}

/* -----------------------------------------------------------------

   FUNCTION:                                extract_ether_address( )

   DESCRIPTION:
      Reads an ethernet address string from the given string, and places it
      in the buffer pointed to by buf.  If no address can be extracted,
      this function returns -1.  On success, returns 0.

   ARGUMENTS:
      char *s - the string to extract from.
      char *val - the extracted address

   RETURN VALUE:
      0 on success, -1 on error.

   ----------------------------------------------------------------- */
int extract_ether_address( char *s, char *buf )
{
   size_t length;
   int i, j;
   int toggle;
   char mask = 0;

   /* Sanity */
   if( ( NULL == s ) || ( NULL == buf ) )
   {
      return -1;
   }
   
   length = strlen( s );
   i = 0;
   toggle = 0;
   /* Loop through the string, looking for characters */
   for( j = 0; j < length; j++ )
   {
      if( s[j] == ':' ) { continue; }
      else if( ( s[j] >= 'a' ) && ( s[j] <= 'f' ) )
      {
         mask = 0x0a + (s[j] - 'a');
      }
      else if( ( s[j] >= 'A' ) && ( s[j] <= 'F' ) )
      {
         mask = 0x0A + (s[j] - 'A');
      }
      else if( ( s[j] >= '0' ) && ( s[j] <= '9' ) )
      {
         mask = 0x00 + (s[j] - '0');
      }
      else
      {
         continue;
      }

      /* Now, see if we need to shift this */
      if( toggle >= 0 )
      {
         buf[i] |= mask;
         if( toggle == 0 )
         {
            buf[i] = buf[i] << 4;
            toggle = 1;
         }
         else
         {
            i++;
            toggle = 0;
         }
      }
      else
      {
         toggle += 2;
      }

      if( i >= ETHER_ADDR_LEN ) { return 0; }
   }
   
   return -1;
}
